﻿{
	// AudioToMarkers.jsx 
	//
	// Developed by Lloyd Alvarez  http://aescripts.com
	// 
	//
	// October 2009
	// 
	// Splits a layer or creates layer markes based on audio threshold
	//
	// v1.2 Added marker numbering option, improved delete marker funciton to work within work area jan 2010
	// v1.1 Fixed undefined alert bug by removing write files pref check nov 2009
	// v1.0 Initial Release oct 2009
	// 
	
	//Globals
function audioToMarkersSetup(thisObj)
	{
	//Define Globals
		var audioToMarkersData = new Object();
		audioToMarkersData.scriptName = "AudioToMarkers";
		audioToMarkersData.scriptVersion = "1.2";
		
		//////////////USER VARIABLES/////////////////////////////////////////////////////////////////////////////////////////
		//if you are not running AE in english you can add a foreign translation here for the "Audio Amplitude"  layer and "Both Channels" effect
		// that are created after running the Convert Audio to Keyframes Keframe Assistant
		audioToMarkersData.audioAmplitudeEffectName = "Audio Amplitude";
		audioToMarkersData.bothChannelsEffectName = "Both Channels";   
		audioToMarkersData.doPurge = true;  //purges image cache before executing,  performance can be sluggish if the images are not purge beforehand
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			
		audioToMarkersData.strHelp = "?";
		audioToMarkersData.thresholdTxt = "Audio Level:";
		audioToMarkersData.thresholdMinDef = 0;
		audioToMarkersData.thresholdMaxDef = 50;
		audioToMarkersData.thresholdDef = 25;
		audioToMarkersData.thresholdHelpTipTxt = "The audio level at which a marker will be placed.";
		audioToMarkersData.resultTxt = "Result:";
		audioToMarkersData.resultOptns = '["Place markers on layer","Split into new layers","Add new text layer"]';
		audioToMarkersData.layerTxt = ["Layer to Mark:","Layer to Split:"];
		audioToMarkersData.layerDef = "Select comp and refresh";
		audioToMarkersData.progressTxt = "";
		audioToMarkersData.progressMax = 100;
		audioToMarkersData.strDoItBtn = ["DO IT!","DO IT!"];
		audioToMarkersData.strRefreshBtn = "Refresh";
		audioToMarkersData.strDoNumberBtn = "Number markers";
		audioToMarkersData.doNumber = false;
		
		audioToMarkersData.languageError = "Since you are not running AE in English you must change the name of the 'Audio Amplitude' layer and 'Both Channels' effect in the variables at the top of the script.\nSimply open the AudioToMarkers.jsx file in any text editor and change both names to what they are in your language";
		audioToMarkersData.compError = "Please select a composition";
		audioToMarkersData.audioLayerError = "Can't find audio amplitude layer\nBefore running this script you must select the audio layer and choose Animation -> Keyframe Assistant -> Convert Audio to Keyframes and leave the newly created layer named to its default 'Audio Amplitude' name.";
		audioToMarkersData.refreshCompError = "After you select your comp click the refresh button to load the comp layers";
		audioToMarkersData.existingMarkersError ="The layer to mark already has markers within the comp work area\nWould you like to delete these markers before proceeding?\n\n(Only markers that are within the work area will be deleted)"
		
		if ($.os.indexOf("Windows") != -1) {var cmd = "CTRL+";} else {var cmd = "Command+";}
		

		audioToMarkersData.strAboutTitle = "About " + audioToMarkersData.scriptName;
		audioToMarkersData.strAbout = audioToMarkersData.scriptName + " v." + audioToMarkersData.scriptVersion + "\n" +
											"Copyright © 2009 Lloyd Alvarez   <http://aescripts.com>\n\n" +
											"Sets markers or splits a layer based on audio intensity. \n\n" + 
											"INSTRUCTIONS:\n"+
											"First you must run the Convert Audio to Keyframes Animation Assitant " +
											"(Animation->Keyframe Assistant->Convert Audio to Keyframes)\n" +
											"on the audio layer you would like to use. \n" +
											"Then set the audio level you would like to trigger the markers, \n" +
											"select the layer to mark or split and press " + audioToMarkersData.strDoItBtn[0] + "\n" +
											"If your comp layers are not loaded, make sure to select the comp first " +
											"then press the " +  audioToMarkersData.strRefreshBtn + " button. \n\n" +
											"Note: The script palette can be used as a dockable panel\nby placing the script in the 'ScriptUI Panels' subfolder of the\nScripts folder (if one doesn't already exist, create one),\nand then choosing this script from the Window menu."
	
		audioToMarkersData.strErrSecurityPrefNotEnabled = "Please enable the \"Allow Scripts to Write Files and Access Network\" option in the General Preferences dialog box, and run this script again.";
		audioToMarkersData.strErrMinAE80 = "This script requires Adobe After Effects CS3 or later.";
		
// UI Functions
		// audioToMarkersSetup_buildUI()
		// Function for creating the user interface
		function audioToMarkersSetup_buildUI(thisObj)
		{	
			var pal = (thisObj instanceof Panel) ? thisObj : new Window("palette", audioToMarkersData.scriptName, undefined, {resizeable:true});
			if (pal != null)
			{
		var res = 
				"group { \
					orientation:'column', alignment:['left','fill'], alignChildren:['left','top'], spacing:5, margins:[0,0,0,0], \
					threshold: Group { \
						alignment:['left','top'], alignChildren:['left','top'], \
						lbl: StaticText { text:'" + audioToMarkersData.thresholdTxt + "', preferredSize: [80,20], helpTip:'" + audioToMarkersData.thresholdHelpTipTxt + "'  }, \
						sldr: Slider { preferredSize: [130,20], helpTip:'" + audioToMarkersData.thresholdHelpTipTxt + "' }, \
						val: StaticText{ text:'" + audioToMarkersData.thresholdDef + "',  preferredSize: [30,20] }, \
					}, \
					result: Group { \
						alignment:['left','top'], alignChildren:['left','top'], \
						lbl: StaticText { text:'" + audioToMarkersData.resultTxt + "', preferredSize: [80,20]}, \
						val: DropDownList { properties:{items:" + audioToMarkersData.resultOptns + "}, alignment:['fill','top'], preferredSize: [164,20]}, \
					}, \
					layer: Group { \
						alignment:['left','top'], alignChildren:['left','top'], \
						lbl: StaticText { text:'" + audioToMarkersData.layerTxt[0] + "', preferredSize: [80,20]}, \
						val: DropDownList {  properties:{items: ['" + audioToMarkersData.layerDef + "']}, alignment:['fill','top'] }, \
					}, \
					refresh: Group { \
						alignment:['fill','top'], alignChildren:['right','top'], \
						refreshBtn: Button {text: '" + audioToMarkersData.strRefreshBtn + "', alignment:['right','top'], preferredSize: [60,20] }, \
						},\
					number: Group { \
						alignment:['left','top'], alignChildren:['left','top'], \
						numberBtn: Checkbox {text: '" + audioToMarkersData.strDoNumberBtn + "'}, \
						},\
					progress: Group { \
						alignment:['fill','top'], alignChildren:['fill','top'], \
						val: Progressbar {  }, \
					}, \
					btns: Group { \
						alignment:['fill','top'], alignChildren:['right','top'], \
						help: Button { text:'" + audioToMarkersData.strHelp + "', alignment:['left','top'] }, \
						doItBtn: Button { text: '" + audioToMarkersData.strDoItBtn[0] + "', alignment:['right','top'], preferredSize: [60,25] }, \
						},\
				}";
			
				
				pal.margins = [10,10,10,10];
				pal.grp = pal.add(res);
				
				var palGfx = pal.graphics;  
				var darkColorBrush = palGfx.newPen(palGfx.BrushType.SOLID_COLOR, [0,0,0], 1); //force the text color to always be black, so it can appear just fine regardless of the UI brightness.
				pal.grp.threshold.sldr.graphics.foregroundColor = darkColorBrush;
				//pal.grp.suffix.val.graphics.foregroundColor = darkColorBrush;
				pal.grp.result.val.graphics.foregroundColor = darkColorBrush;
				pal.grp.layer.val.graphics.foregroundColor = darkColorBrush;
				pal.grp.btns.help.preferredSize.width = pal.grp.btns.help.maximumSize.width = pal.grp.btns.help.minimumSize.width = 30;
				pal.grp.btns.help.onClick = function () {alert(audioToMarkersData.strAbout, audioToMarkersData.strAboutTitle);}
				
				pal.grp.btns.doItBtn.onClick = function () {
						var stopProcess = false;
						/*if (this.text == audioToMarkersData.strDoItBtn[1] ) {
							var stopProcess = true;
							}*/
						doIt(pal,stopProcess);
					}
				/*var myStop = false;
				pal.grp.btns.cancelBtn.onClick = function () {myStop = true;} //stop process if Stop button is clicked
				*/
				pal.layout.layout(true);
				pal.grp.minimumSize = pal.grp.size;
				
				pal.grp.threshold.sldr.minvalue = audioToMarkersData.thresholdMinDef;
				pal.grp.threshold.sldr.maxvalue = audioToMarkersData.thresholdMaxDef;
				pal.grp.threshold.sldr.value = audioToMarkersData.thresholdDef;
				pal.grp.threshold.val.text = pal.grp.threshold.sldr.value;
				pal.grp.number.numberBtn.value = audioToMarkersData.doNumber;
				pal.grp.result.val.selection = 0;
				pal.grp.progress.hide();
				
				loadLayers(pal,0);
				pal.grp.layer.val.selection = 0;
				
				//alert (audioToMarkersData.layerDef);
				if (pal.grp.layer.val.selection.text == audioToMarkersData.layerDef) {
					pal.grp.btns.doItBtn.enabled = false;
					} else {
					pal.grp.btns.doItBtn.enabled = true;	
						}
				
				pal.grp.layer.lbl.text = audioToMarkersData.layerTxt[pal.grp.result.val.selection.index];
				
				pal.grp.number.numberBtn.onClick = function () {
					audioToMarkersData.doNumber = this.value;
					}
				
				pal.grp.result.val.onChange = function () {
					pal.grp.layer.visible = true;
					pal.grp.layer.lbl.text = audioToMarkersData.layerTxt[pal.grp.result.val.selection.index];
				
				if (pal.grp.layer.val.selection.text == audioToMarkersData.layerDef) {
						pal.grp.btns.doItBtn.enabled = false;
						} else {
						pal.grp.btns.doItBtn.enabled = true;	
							}
						
					if (this.selection.index == 2) {
						pal.grp.layer.visible = false;
						} 
					}
				
				pal.grp.refresh.refreshBtn.onClick = function () {
					loadLayers(pal,1);
					}
				

				pal.grp.threshold.sldr.onChanging = function () {
					this.parent.parent.threshold.val.text = this.parent.parent.threshold.sldr.value;
					}		
			}		
			return pal;
		}
	
	//function:
	

	
	function doIt (atmPalObj,stopProcess) {
		if (audioToMarkersData.doPurge) app.purge(PurgeTarget.IMAGE_CACHES);
		var myComp = app.project.activeItem;
		if (myComp == null || !(myComp instanceof CompItem)){ 
			if (stage != 0) {
			alert(audioToMarkersData.compError ); 
			atmPalObj.grp.btns.doItBtn.enabled = false;
			atmPalObj.grp.layer.val.removeAll();
			atmPalObj.grp.layer.val.add("item",audioToMarkersData.layerDef);
			atmPalObj.grp.layer.val.selection = 0;
			}
			return;
			} //if (myComp == null || !(myComp instanceof CompItem))
		
		var thres = atmPalObj.grp.threshold.sldr.value;
		var layerName = atmPalObj.grp.layer.val.selection.text
		var layerIndex = parseInt(layerName.match(/^[0-9]+/),10);
		var markLayer = myComp.layer(layerIndex);
		var audioLayer = preFlight(atmPalObj,1);
		if (audioLayer != null) {
			//var minMax = getMinMaxAmp (myComp, audioLayer);
			app.beginUndoGroup(audioToMarkersData.scriptName);
			var progBar = atmPalObj.grp.progress.val;
			var resultAction = atmPalObj.grp.result.val.selection.index;
			atmPalObj.grp.btns.doItBtn.text = audioToMarkersData.strDoItBtn[1];
			atmPalObj.grp.progress.show();
			while (!stopProcess) {
			stopProcess = setMarkers (progBar,thres,myComp, markLayer, audioLayer,resultAction,stopProcess);
			}
			atmPalObj.grp.progress.hide();
			atmPalObj.grp.btns.doItBtn.text = audioToMarkersData.strDoItBtn[0];
			app.endUndoGroup();
			
			} //if (audioLayer)
		} //doIt ()
	
	function loadLayers (atmPalObj,stage) {
		var myComp = app.project.activeItem;
		if (myComp == null || !(myComp instanceof CompItem)){ 
			if (stage != 0) {
			alert(audioToMarkersData.compError ); 
			atmPalObj.grp.layer.val.removeAll();
			atmPalObj.grp.layer.val.add("item",audioToMarkersData.layerDef);
			atmPalObj.grp.layer.val.selection = 0;
			atmPalObj.grp.btns.doItBtn.enabled = false;
			}
			return;
			} //if (myComp == null || !(myComp instanceof CompItem))
		
		var audioLayer = preFlight(atmPalObj,1);
		if (audioLayer == null) {
			return
			}
		loadMinMax(atmPalObj,myComp,audioLayer);
		var dropDown = atmPalObj.grp.layer.val;
		dropDown.removeAll();
		for (var i=1; i <= myComp.numLayers; i++) {
			dropDown.add("item",myComp.layer(i).index + ". " +myComp.layer(i).name);
			} //for i
		dropDown.selection = 0;
		
		
		}//function loadLayers
	
	
	function preFlight (atmPalObj,stage) {
		stage = stage || 1;
		clearOutput();

		var myComp = app.project.activeItem;
		if (myComp == null || !(myComp instanceof CompItem)){ 
				atmPalObj.grp.btns.doItBtn.enabled = false;
				atmPalObj.grp.layer.val.removeAll();
				atmPalObj.grp.layer.val.add("item",audioToMarkersData.layerDef);
				atmPalObj.grp.layer.val.selection = 0;
				//alert(audioToMarkersData.compError ); 
				return null;
				}
			
	if (stage == 1) {
		var audioLayer = findAudioLayer(myComp);
		if (audioLayer == null) {
		alert(audioToMarkersData.audioLayerError ); 
		atmPalObj.grp.btns.doItBtn.enabled = false;
				return null;
				}
		}
		atmPalObj.grp.btns.doItBtn.enabled = true;
		return audioLayer;
		} //preFlight ()
	
	function findAudioLayer (comp)  {
        var audioAmpLayer = null;
		if (app.language != Language.ENGLISH) {
			
			
			}
		
        for (var i = 1; !audioAmpLayer && i <= comp.numLayers; i++)  {
            var curLayer = comp.layer(i);
            if (curLayer.name == audioToMarkersData.audioAmplitudeEffectName)  {
                var err = false;
                try    {
                    // check existence of Both Channels effect
                    curLayer.effect(audioToMarkersData.bothChannelsEffectName).name = curLayer.effect(audioToMarkersData.bothChannelsEffectName).name;                    
                }
                catch(e)  {
                    err = true;
                }
                if (!err)  {               
                    audioAmpLayer = curLayer;
                }
            }
        }
        
        return audioAmpLayer;
    }

     function loadMinMax (atmPalObj,comp, layer){
        var min = 1e6;
        var max = -min;
		var prop = layer.effect(audioToMarkersData.bothChannelsEffectName).property(1);
        for (var t = Math.max(Math.max(comp.displayStartTime,layer.inPoint),comp.workAreaStart); t < Math.min(Math.min(comp.duration,layer.outPoint),comp.workAreaStart +comp.workAreaDuration); t += comp.frameDuration)
        {
            var curAmp = prop.valueAtTime(t, false); 
            if (curAmp < min) min = curAmp;
            if (curAmp > max) max = curAmp;
        }  
	
		atmPalObj.grp.threshold.sldr.minvalue = min;
		atmPalObj.grp.threshold.sldr.maxvalue = max;
		if (atmPalObj.grp.threshold.sldr.value < min || atmPalObj.grp.threshold.sldr.value > max) {
		atmPalObj.grp.threshold.sldr.value = (max/2);
		atmPalObj.grp.threshold.val.text = atmPalObj.grp.threshold.sldr.value;
        }
	
        return [min,max];
    }


function deleteMarkers (marker,comp,layer) 
{
		var workAreaIn = Math.max(Math.max(comp.displayStartTime,layer.inPoint),comp.workAreaStart);
		var workAreaOut = Math.min(Math.min(comp.duration,layer.outPoint),comp.workAreaStart +comp.workAreaDuration);
		
		for (var i=marker.numKeys; i > 0; i--) {
			if (marker.keyTime(i) >= workAreaIn && marker.keyTime(i) <= workAreaOut) {
				marker.removeKey(i);
			}
		}
 }

function checkForMarkers (marker,comp,layer)
{
	var workAreaIn = Math.max(Math.max(comp.displayStartTime,layer.inPoint),comp.workAreaStart);
	var workAreaOut = Math.min(Math.min(comp.duration,layer.outPoint),comp.workAreaStart +comp.workAreaDuration);
	for (var i=1; i <= marker.numKeys; i++) {
			if (marker.keyTime(i) >= workAreaIn && marker.keyTime(i) <= workAreaOut) {
				return true;
			}
		}
	return false;
}

function setMarkers (progBar,thres,comp, mLayer,layer,resultAction,stopProcess)  {
		var prop = layer.effect(audioToMarkersData.bothChannelsEffectName).property(1);
        var counter = 1;
		var overThres = false;
		
		if (checkForMarkers(mLayer.property("Marker"),comp,layer) && resultAction != 2) {
			var deleteMkrs = false;
			deleteMkrs = confirm (audioToMarkersData.existingMarkersError);
				if (deleteMkrs) {
					deleteMarkers (mLayer.property("Marker"),comp,layer);
				}
			}
		if (resultAction == 2) {
		var myTextLayer = comp.layers.addText(audioToMarkersData.scriptName);
		var myTextProp = myTextLayer.property("Source Text");
		myTextLayer.name = audioToMarkersData.scriptName;
		}
		  for (var t = Math.max(Math.max(comp.displayStartTime,layer.inPoint),comp.workAreaStart); t < Math.min(Math.min(comp.duration,layer.outPoint),comp.workAreaStart +comp.workAreaDuration) || stopProcess; t += comp.frameDuration)
        {
			progBar.value = t;
			progBar.maxvalue = Math.min(comp.duration,layer.outPoint);
			progBar.value++;  //update the progress bar in the UI ***currently not working on Mac CS3, bug reported
		
			if (parseFloat(app.version) >= 9) {
			progBar.window.update();  // This call refreshes the UI and fixes the Mac CS3 bug, but works in CS4 only.
			}
			clearOutput();
			writeLn ("AudioToMarkers completion " + (t/Math.min(comp.duration,layer.outPoint)).toFixed(2) *100+ "%");
			//alert (curAmp + " " + thres);
            var curAmp = prop.valueAtTime(t, false); 
			if ( curAmp > thres && !overThres) {
				//alert (resultAction);
			switch (resultAction) {
				case 0:
				
							mLayer.property("Marker").setValueAtTime(t, new MarkerValue((audioToMarkersData.doNumber)?counter:""));
							counter++;
							break;
				case 1: //split layer
							//this is what we do if the split layers pref is set, easy since the time marker is at the current time of the edit
							var saveOutPoint = mLayer.outPoint;  //first we need to save the original outPoint of the layer, I'll explain why below
							mLayer.duplicate();
							mLayer.inPoint = t;     //makes no sense to but when I set the inPoint it also sets the outPoint at the same time making a 1 frame long layer
							mLayer.outPoint = saveOutPoint;  //so I have to reset the outPoint the original place..  this smells like a bug to me.
							comp.layer(mLayer.index-1).outPoint = t;
							break;
				case 2: // create text layer
							myTextProp.setValueAtTime(t, new TextDocument((audioToMarkersData.doNumber)?counter:""));
							counter++;
							break;
				
				} //switch (resultAction)
			
			overThres = true;
			}
			if (curAmp < thres ) {
				overThres = false;
				}
		
        }  
		clearOutput();
        return true;
    }
	
		// main:
		// 

		if (parseFloat(app.version) < 8)
		{
			alert(audioToMarkersData.MinAE80Error , audioToMarkersData.scriptName);
			return;
		}
		else
		{
		
		if ( app.language != Language.ENGLISH &&  (audioToMarkersData.audioAmplitudeEffectName == "Audio Amplitude" || audioToMarkersData.bothChannelsEffectName == "Both Channels") ) {
			alert(audioToMarkersData.languageError);
			return;
			}
		
			
			var audioToMarkersPal = audioToMarkersSetup_buildUI(thisObj);
			if ((audioToMarkersPal != null) && (audioToMarkersPal instanceof Window))
			{
				audioToMarkersPal.center();
				audioToMarkersPal.show();
			}
		}
	}
	
	audioToMarkersSetup(this);
	
	
	}